use async_graphql::{ComplexObject, Context, FieldResult, SimpleObject};
use sqlx::PgPool;

use crate::{models::gruppe::Gruppe, scalar::Id};

use super::rolle::Rolle;

#[derive(SimpleObject)]
#[graphql(complex)]
pub struct Benutzer {
    /// Die UUID eines Benutzers
    pub id: Id,

    /// Die Kennung des Benutzers
    pub kennung: String,

    /// Der Vorname des Benutzers
    pub vorname: String,

    /// Der Nachname des Benutzers
    pub nachname: String,
}

#[ComplexObject]
impl Benutzer {
    /// Die Rollen des Benutzers
    pub async fn rollen<'ctx>(&self, ctx: &Context<'ctx>) -> FieldResult<Vec<Rolle>> {
        let pool = ctx.data::<PgPool>()?;

        let rows = sqlx::query_as!(
            Rolle,
            r#"
                SELECT
                    r.id,
                    r.rollenname,
                    r.erstellt_am,
                    r.geaendert_am
                FROM rollen AS r
                LEFT JOIN benutzer_rollen AS br ON r.id = br.rolle_id
                WHERE br.benutzer_id = $1;
            "#,
            &self.id
        )
        .fetch_all(pool)
        .await?;

        Ok(rows)
    }

    /// Die Gruppen des Benutzers
    pub async fn gruppen<'ctx>(&self, ctx: &Context<'ctx>) -> FieldResult<Vec<Gruppe>> {
        let pool = ctx.data::<PgPool>()?;

        let rows = sqlx::query_as!(
            Gruppe,
            r#"
                SELECT
                    g.id,
                    g.gruppenname,
                    g.erstellt_am,
                    g.geaendert_am
                FROM gruppen AS g
                LEFT JOIN benutzer_gruppen AS bg ON g.id = bg.gruppe_id
                WHERE bg.benutzer_id = $1;
            "#,
            &self.id
        )
        .fetch_all(pool)
        .await?;

        Ok(rows)
    }
}
