use async_graphql::{ComplexObject, Context, FieldResult, SimpleObject};
use sqlx::PgPool;

use crate::scalar::{Id, Time};

use super::gruppe::Gruppe;

/// Um die Administration zu erleichtern werden Gruppen in die Rollen hinzugefuegt
#[derive(SimpleObject, Debug)]
#[graphql(complex)]
pub struct Rolle {
    /// Die  uuid einer Rolle
    pub id: Id,

    /// Der Rollenname
    pub rollenname: String,

    /// Wann die Rolle erstellt wurde
    pub erstellt_am: Time,

    /// Wann die Rolle geaendert wurde
    pub geaendert_am: Time,
}

#[ComplexObject]
impl Rolle {
    /// Die Gruppen in einer Rolle
    pub async fn gruppen<'ctx>(&self, ctx: &Context<'ctx>) -> FieldResult<Vec<Gruppe>> {
        let pool = ctx.data::<PgPool>()?;

        let rows = sqlx::query_as!(
            Gruppe,
            r#"
                SELECT
                    g.id,
                    g.gruppenname,
                    g.erstellt_am,
                    g.geaendert_am
                FROM gruppen AS g
                LEFT JOIN rollen_gruppen AS rg ON g.id = rg.gruppe_id
                WHERE rg.rolle_id = $1;
            "#,
            &self.id
        )
        .fetch_all(pool)
        .await?;

        Ok(rows)
    }
}
